@@ -12,9 +12,11 @@ from __future__ import print_function, division
 import warnings
 import json
 import time
+import string
 import os
 import re
 import keyring
+import threading
 
 import numpy as np
 
@@ -28,6 +30,7 @@ import astropy.coordinates as coord
 from astropy.table import Table, Row, vstack, MaskedColumn
 from astropy.extern.six.moves.urllib.parse import quote as urlencode
 from astropy.extern.six.moves.http_cookiejar import Cookie
+from astropy.utils.console import ProgressBarOrSpinner
 from astropy.utils.exceptions import AstropyWarning
 from astropy.logger import log
 
@@ -39,7 +42,6 @@ from ..exceptions import (TimeoutError, InvalidQueryError, RemoteServiceError,
                           NoResultsWarning, InputWarning, AuthenticationWarning)
 from . import conf
 
-
 __all__ = ['Observations', 'ObservationsClass',
            'Mast', 'MastClass']
 
@@ -104,6 +106,7 @@ def _mashup_json_to_table(json_obj, col_config=None):
         if atype == "boolean":
             atype = "bool"
         if atype == "int":  # int arrays do not admit Non/nan vals
+            atype = np.int64
             ignoreValue = -999 if (ignoreValue is None) else ignoreValue
 
         # Make the column list (don't assign final type yet or there will be errors)
@@ -149,7 +152,7 @@ class MastClass(QueryWithLogin):
         self.TIMEOUT = conf.timeout
         self.PAGESIZE = conf.pagesize
 
-        self._column_configs = {}
+        self._column_configs = dict()
         self._current_service = None
 
         if username or session_token:
@@ -388,7 +391,7 @@ class MastClass(QueryWithLogin):
 
         return allResponses
 
-    def _get_col_config(self, service):
+    def _get_col_config(self, service, fetch_name=None):
         """
         Gets the columnsConfig entry for given service and stores it in `self._column_configs`.
 
@@ -396,14 +399,20 @@ class MastClass(QueryWithLogin):
         ----------
         service : string
             The service for which the columns config will be fetched.
+        fetch_name : string, optional
+            If the columns-config associated with the service has a different name,
+            use this argument. The default sets it to the same as service.
         """
 
+        if not fetch_name:
+            fetch_name = service
+
         headers = {"User-Agent": self._session.headers["User-Agent"],
                    "Content-type": "application/x-www-form-urlencoded",
                    "Accept": "text/plain"}
 
         response = self._request("POST", self._COLUMNS_CONFIG_URL,
-                                 data=("colConfigId="+service), headers=headers)
+                                 data=("colConfigId="+fetch_name), headers=headers)
 
         self._column_configs[service] = response[0].json()
 
@@ -702,46 +711,16 @@ class MastClass(QueryWithLogin):
 
         return coordinates
 
-
-@async_to_sync
-class ObservationsClass(MastClass):
-    """
-    MAST Observations query class.
-
-    Class for querying MAST observational data.
-    """
-
-    def list_missions(self):
-        """
-        Lists data missions archived by MAST and avaiable through `astroquery.mast`.
-
-        Returns
-        --------
-        response : list
-            List of available missions.
-        """
-
-        # getting all the hitogram information
-        service = "Mast.Caom.All"
-        params = {}
-        response = Mast.service_request_async(service, params, format='extjs')
-        jsonResponse = response[0].json()
-
-        # getting the list of missions
-        histData = jsonResponse['data']['Tables'][0]['Columns']
-        for facet in histData:
-            if facet['text'] == "obs_collection":
-                missionInfo = facet['ExtendedProperties']['histObj']
-                missions = list(missionInfo.keys())
-                missions.remove('hist')
-                return missions
-
-    def _build_filter_set(self, **filters):
+    def _build_filter_set(self, column_config_name, service_name=None, **filters):
         """
         Takes user input dictionary of filters and returns a filterlist that the Mashup can understand.
 
         Parameters
         ----------
+        column_config_name : string
+            The service for which the columns config will be fetched.
+        service_name : string, optional
+            The service that will use the columns config, default is to be the same as column_config_name.
         **filters :
             Filters to apply. At least one filter must be supplied.
             Valid criteria are coordinates, objectname, radius (as in `query_region` and `query_object`),
@@ -756,10 +735,13 @@ class ObservationsClass(MastClass):
             The mashup json filter object.
         """
 
-        if not self._column_configs.get("Mast.Caom.Cone"):
-            self._get_col_config("Mast.Caom.Cone")
+        if not service_name:
+            service_name = column_config_name
 
-        caomColConfig = self._column_configs["Mast.Caom.Cone"]
+        if not self._column_configs.get(service_name):
+            self._get_col_config(service_name, fetch_name=column_config_name)
+
+        caomColConfig = self._column_configs[service_name]
 
         mashupFilters = []
         for colname, value in filters.items():
@@ -775,7 +757,7 @@ class ObservationsClass(MastClass):
                 continue
 
             colType = "discrete"
-            if colInfo.get("vot.datatype", colInfo.get("type")) in ("double", "float"):
+            if (colInfo.get("vot.datatype", colInfo.get("type")) in ("double", "float")) or colInfo.get("treatNumeric"):
                 colType = "continuous"
 
             separator = colInfo.get("separator")
@@ -827,6 +809,47 @@ class ObservationsClass(MastClass):
 
         return mashupFilters
 
+
+@async_to_sync
+class ObservationsClass(MastClass):
+    """
+    MAST Observations query class.
+
+    Class for querying MAST observational data.
+    """
+
+    def __init__(self, *args, **kwargs):
+
+        super(ObservationsClass, self).__init__(*args, **kwargs)
+
+        self._boto3 = None
+        self._botocore = None
+
+    def list_missions(self):
+        """
+        Lists data missions archived by MAST and avaiable through `astroquery.mast`.
+
+        Returns
+        --------
+        response : list
+            List of available missions.
+        """
+
+        # getting all the hitogram information
+        service = "Mast.Caom.All"
+        params = {}
+        response = Mast.service_request_async(service, params, format='extjs')
+        jsonResponse = response[0].json()
+
+        # getting the list of missions
+        histData = jsonResponse['data']['Tables'][0]['Columns']
+        for facet in histData:
+            if facet['text'] == "obs_collection":
+                missionInfo = facet['ExtendedProperties']['histObj']
+                missions = list(missionInfo.keys())
+                missions.remove('hist')
+                return missions
+
     @class_or_instance
     def query_region_async(self, coordinates, radius=0.2*u.deg, pagesize=None, page=None):
         """
@@ -944,8 +967,11 @@ class ObservationsClass(MastClass):
         # grabbing the observation type (science vs calibration)
         obstype = criteria.pop('obstype', 'science')
 
-        # Build the mashup filter object
-        mashupFilters = self._build_filter_set(**criteria)
+        # Build the mashup filter object and store it in the correct service_name entry
+        if coordinates or objectname:
+            mashupFilters = self._build_filter_set("Mast.Caom.Cone", "Mast.Caom.Filtered.Position", **criteria)
+        else:
+            mashupFilters = self._build_filter_set("Mast.Caom.Cone", "Mast.Caom.Filtered", **criteria)
 
         if not mashupFilters:
             raise InvalidQueryError("At least one non-positional criterion must be supplied.")
@@ -1094,8 +1120,11 @@ class ObservationsClass(MastClass):
         # grabbing the observation type (science vs calibration)
         obstype = criteria.pop('obstype', 'science')
 
-        # Build the mashup filter object
-        mashupFilters = self._build_filter_set(**criteria)
+        # Build the mashup filter object and store it in the correct service_name entry
+        if coordinates or objectname:
+            mashupFilters = self._build_filter_set("Mast.Caom.Cone", "Mast.Caom.Filtered.Position", **criteria)
+        else:
+            mashupFilters = self._build_filter_set("Mast.Caom.Cone", "Mast.Caom.Filtered", **criteria)
 
         # handle position info (if any)
         position = None
@@ -1270,6 +1299,132 @@ class ObservationsClass(MastClass):
                           "URL": [url]})
         return manifest
 
+    def enable_s3_hst_dataset(self):
+        """
+        Attempts to enable downloading HST public files from S3 instead of MAST.
+        Requires the boto3 library to function.
+        """
+        import boto3
+        import botocore
+        self._boto3 = boto3
+        self._botocore = botocore
+
+        log.info("Using the S3 HST public dataset")
+        log.warning("Your AWS account will be charged for access to the S3 bucket")
+        log.info("See Request Pricing in https://aws.amazon.com/s3/pricing/ for details")
+        log.info("If you have not configured boto3, follow the instructions here: " +
+                 "https://boto3.readthedocs.io/en/latest/guide/configuration.html")
+
+    def disable_s3_hst_dataset(self):
+        """
+        Disables downloading HST public files from S3 instead of MAST
+        """
+        self._boto3 = None
+        self._botocore = None
+
+    def _download_from_s3(self, dataProduct, localPath, cache=True):
+        # The following is a mishmash of BaseQuery._download_file and s3 access through boto
+
+        bkt_name = 'stpubdata'
+
+        # This is a cheap operation and does not perform any actual work yet
+        s3 = self._boto3.resource('s3')
+        s3_client = self._boto3.client('s3')
+        bkt = s3.Bucket(bkt_name)
+
+        dataUri = dataProduct['dataURI']
+        filename = dataUri.split("/")[-1]
+        obs_id = dataProduct['obs_id']
+
+        obs_id = obs_id.lower()
+
+        # This next part is a bit funky.  Let me explain why:
+        # We have 2 different possible URI schemes for HST:
+        #   mast:HST/product/obs_id_filename.type (old style)
+        #   mast:HST/product/obs_id/obs_id_filename.type (new style)
+        # The first scheme was developed thinking that the obs_id in the filename
+        # would *always* match the actual obs_id folder the file was placed in.
+        # Unfortunately this assumption was false.
+        # We have been trying to switch to the new uri scheme as it specifies the
+        # obs_id used in the folder path correctly.
+        # The cherry on top is that the obs_id in the new style URI is not always correct either!
+        # When we are looking up files we have some code which iterates through all of
+        # the possible permutations of the obs_id's last char which can be *ANYTHING*
+        #
+        # So in conclusion we can't trust the last char obs_id from the file or from the database
+        # So with that in mind, hold your nose when reading the following:
+
+        info_lookup = None
+        sane_path = os.path.join("hst", "public", obs_id[:4], obs_id, filename)
+        try:
+            info_lookup = s3_client.head_object(Bucket=bkt_name, Key=sane_path, RequestPayer='requester')
+            bucketPath = sane_path
+        except self._botocore.exceptions.ClientError as e:
+            if e.response['Error']['Code'] != "404":
+                raise
+
+        if info_lookup is None:
+            # Unfortunately our file placement logic is anything but sane
+            # We put files in folders that don't make sense
+            for ch in (string.digits + string.ascii_lowercase):
+                # The last char of the obs_folder (observation id) can be any lowercase or numeric char
+                insane_obs = obs_id[:-1] + ch
+                insane_path = os.path.join("hst", "public", insane_obs[:4], insane_obs, filename)
+
+                try:
+                    info_lookup = s3_client.head_object(Bucket=bkt_name, Key=insane_path, RequestPayer='requester')
+                    bucketPath = insane_path
+                    break
+                except self._botocore.exceptions.ClientError as e:
+                    if e.response['Error']['Code'] != "404":
+                        raise
+
+        if info_lookup is None:
+            raise Exception("Unable to locate file!")
+
+        # Unfortunately, we can't use the reported file size in the reported product.  STScI's backing
+        # archive database (CAOM) is frequently out of date and in many cases omits the required information.
+        # length = dataProduct["size"]
+        # Instead we ask the webserver (in this case S3) what the expected content length is and use that.
+        length = info_lookup["ContentLength"]
+
+        if cache and os.path.exists(localPath):
+            if length is not None:
+                statinfo = os.stat(localPath)
+                if statinfo.st_size != length:
+                    log.warning("Found cached file {0} with size {1} that is "
+                                "different from expected size {2}"
+                                .format(localPath,
+                                        statinfo.st_size,
+                                        length))
+                else:
+                    log.info("Found cached file {0} with expected size {1}."
+                             .format(localPath, statinfo.st_size))
+                    return
+
+        with ProgressBarOrSpinner(length, ('Downloading URL s3://{0}/{1} to {2} ...'.format(
+                bkt_name, bucketPath, localPath))) as pb:
+
+            # Bytes read tracks how much data has been received so far
+            # This variable will be updated in multiple threads below
+            global bytes_read
+            bytes_read = 0
+
+            progress_lock = threading.Lock()
+
+            def progress_callback(numbytes):
+                # Boto3 calls this from multiple threads pulling the data from S3
+                global bytes_read
+
+                # This callback can be called in multiple threads
+                # Access to updating the console needs to be locked
+                with progress_lock:
+                    bytes_read += numbytes
+                    pb.update(bytes_read)
+
+            bkt.download_file(bucketPath, localPath, ExtraArgs={"RequestPayer": "requester"},
+                              Callback=progress_callback)
+
     def _download_files(self, products, base_dir, cache=True):
         """
         Takes an `astropy.table.Table` of data products and downloads them into the dirctor given by base_dir.
@@ -1292,7 +1447,7 @@ class ObservationsClass(MastClass):
         for dataProduct in products:
 
             localPath = base_dir + "/" + dataProduct['obs_collection'] + "/" + dataProduct['obs_id']
-            dataUrl = self._MAST_DOWNLOAD_URL + "?uri=" + dataProduct['dataURI']
+            dataUrl = self._MAST_DOWNLOAD_URL + "?uri=" + dataProduct["dataURI"]
 
             if not os.path.exists(localPath):
                 os.makedirs(localPath)
@@ -1304,20 +1459,23 @@ class ObservationsClass(MastClass):
             url = None
 
             try:
-                self._download_file(dataUrl, localPath, cache=cache)
+                if self._boto3 is not None and dataProduct["dataURI"].startswith("mast:HST/product"):
+                    try:
+                        self._download_from_s3(dataProduct, localPath, cache)
+                    except Exception as ex:
+                        log.exception("Error pulling from S3 bucket: %s" % ex)
+                        log.warn("Falling back to mast download...")
+                        self._download_file(dataUrl, localPath, cache=cache)
+                else:
+                    self._download_file(dataUrl, localPath, cache=cache)
 
-                # check file size also this is where would perform md5
+                # check if file exists also this is where would perform md5,
+                # and also check the filesize if the database reliably reported file sizes
                 if not os.path.isfile(localPath):
                     status = "ERROR"
                     msg = "File was not downloaded"
                     url = dataUrl
-                else:
-                    fileSize = os.stat(localPath).st_size
-                    if fileSize != dataProduct["size"]:
-                        status = "ERROR"
-                        msg = "Downloaded filesize is {},".format(dataProduct['size']) + \
-                              "but should be {}, file may be partial or corrupt.".format(fileSize)
-                        url = dataUrl
+
             except HTTPError as err:
                 status = "ERROR"
                 msg = "HTTPError: {0}".format(err)
@@ -1399,5 +1557,417 @@ class ObservationsClass(MastClass):
         return manifest
 
 
+@async_to_sync
+class CatalogsClass(MastClass):
+    """
+    MAST catalog query class.
+
+    Class for querying MAST catalog data.
+    """
+
+    def __init__(self):
+
+        super(CatalogsClass, self).__init__()
+
+        self.catalogLimit = None
+
+    def _parse_result(self, response, verbose=False):
+
+        resultsTable = super(CatalogsClass, self)._parse_result(response, verbose)
+
+        if len(resultsTable) == self.catalogLimit:
+            warnings.warn("Maximum catalog results returned, may not include all sources within radius.",
+                          MaxResultsWarning)
+
+        return resultsTable
+
+    @class_or_instance
+    def query_region_async(self, coordinates, radius=0.2*u.deg, catalog="Hsc",
+                           pagesize=None, page=None, **kwargs):
+        """
+        Given a sky position and radius, returns a list of catalog entries.
+        See column documentation for specific catalogs `here <https://mast.stsci.edu/api/v0/pages.htmll>`__.
+
+        Parameters
+        ----------
+        coordinates : str or `astropy.coordinates` object
+            The target around which to search. It may be specified as a
+            string or as the appropriate `astropy.coordinates` object.
+        radius : str or `~astropy.units.Quantity` object, optional
+            Default 0.2 degrees.
+            The string must be parsable by `astropy.coordinates.Angle`. The
+            appropriate `~astropy.units.Quantity` object from
+            `astropy.units` may also be used. Defaults to 0.2 deg.
+        catalog : str, optional
+            Default HSC.
+            The catalog to be queried.
+        pagesize : int, optional
+            Default None.
+            Can be used to override the default pagesize for (set in configs) this query only.
+            E.g. when using a slow internet connection.
+        page : int, optional
+            Default None.
+            Can be used to override the default behavior of all results being returned to
+            obtain a specific page of results.
+        **kwargs
+            Catalog-specific keyword args.
+            These can be found in the (service documentation)[https://mast.stsci.edu/api/v0/_services.html]
+            for specific catalogs. For example one can specify the magtype for an HSC search.
+
+        Returns
+        -------
+        response: list of ``requests.Response``
+        """
+
+        # Put coordinates and radius into consistant format
+        coordinates = commons.parse_coordinates(coordinates)
+
+        # if radius is just a number we assume degrees
+        if isinstance(radius, (int, float)):
+            radius = radius * u.deg
+        radius = coord.Angle(radius)
+
+        # Figuring out the service
+        if catalog.lower() == "hsc":
+            service = "Mast.Hsc.Db"
+            self.catalogLimit = kwargs.get('nr', 50000)
+        elif catalog.lower() == "galex":
+            service = "Mast.Galex.Catalog"
+            self.catalogLimit = kwargs.get('maxrecords', 50000)
+        else:
+            service = "Mast.Catalogs." + catalog + ".Cone"
+            self.catalogLimit = None
+
+        # basic params
+        params = {'ra': coordinates.ra.deg,
+                  'dec': coordinates.dec.deg,
+                  'radius': radius.deg}
+
+        # Hsc specific parameters (can be overridden by user)
+        params['nr'] = 50000
+        params['ni'] = 1
+        params['magtype'] = 1
+
+        # galex specific parameters (can be overridden by user)
+        params['maxrecords'] = 50000
+
+        # adding additional parameters
+        for prop, value in kwargs.items():
+            params[prop] = value
+
+        return self.service_request_async(service, params, pagesize, page)
+
+    @class_or_instance
+    def query_object_async(self, objectname, radius=0.2*u.deg, catalog="Hsc",
+                           pagesize=None, page=None, **kwargs):
+        """
+        Given an object name, returns a list of catalog entries.
+        See column documentation for specific catalogs `here <https://mast.stsci.edu/api/v0/pages.html>`__.
+
+        Parameters
+        ----------
+        objectname : str
+            The name of the target around which to search.
+        radius : str or `~astropy.units.Quantity` object, optional
+            Default 0.2 degrees.
+            The string must be parsable by `astropy.coordinates.Angle`.
+            The appropriate `~astropy.units.Quantity` object from
+            `astropy.units` may also be used. Defaults to 0.2 deg.
+        catalog : str, optional
+            Default HSC.
+            The catalog to be queried.
+        pagesize : int, optional
+            Default None.
+            Can be used to override the default pagesize for (set in configs) this query only.
+            E.g. when using a slow internet connection.
+        page : int, optional
+            Defaulte None.
+            Can be used to override the default behavior of all results being returned
+            to obtain a specific page of results.
+        **kwargs
+            Catalog-specific keyword args.
+            These can be found in the `service documentation <https://mast.stsci.edu/api/v0/_services.html>`__.
+            for specific catalogs. For example one can specify the magtype for an HSC search.
+
+        Returns
+        -------
+        response: list of ``requests.Response``
+        """
+
+        coordinates = self._resolve_object(objectname)
+
+        return self.query_region_async(coordinates, radius, catalog, pagesize, page, **kwargs)
+
+    @class_or_instance
+    def query_criteria_async(self, catalog, pagesize=None, page=None, **criteria):
+        """
+        Given an set of filters, returns a list of catalog entries.
+        See column documentation for specific catalogs `here <https://mast.stsci.edu/api/v0/pages.htmll>`__.
+
+        Parameters
+        ----------
+        pagesize : int, optional
+            Can be used to override the default pagesize.
+            E.g. when using a slow internet connection.
+        page : int, optional
+            Can be used to override the default behavior of all results being returned to obtain
+            one specific page of results.
+        **criteria
+            Criteria to apply. At least one non-positional criteria must be supplied.
+            Valid criteria are coordinates, objectname, radius (as in `query_region` and `query_object`),
+            and all observation fields listed `here <https://mast.stsci.edu/api/v0/_c_a_o_mfields.html>`__.
+            The Column Name is the keyword, with the argument being one or more acceptable values for that parameter,
+            except for fields with a float datatype where the argument should be in the form [minVal, maxVal].
+            For non-float type criteria wildcards maybe used (both * and % are considered wildcards), however
+            only one wildcarded value can be processed per criterion.
+            RA and Dec must be given in decimal degrees, and datetimes in MJD.
+            For example: filters=["FUV","NUV"],proposal_pi="Ost*",t_max=[52264.4586,54452.8914]
+
+
+        Returns
+        -------
+        response : list(`requests.Response`)
+        """
+
+        # Seperating any position info from the rest of the filters
+        coordinates = criteria.pop('coordinates', None)
+        objectname = criteria.pop('objectname', None)
+        radius = criteria.pop('radius', 0.2*u.deg)
+
+        # Build the mashup filter object
+        if catalog == "Tic":
+            service = "Mast.Catalogs.Filtered.Tic"
+            if coordinates or objectname:
+                service += ".Position"
+            mashupFilters = self._build_filter_set("Mast.Catalogs.Tess.Cone", service, **criteria)
+
+        elif catalog == "DiskDetective":
+            service = "Mast.Catalogs.Filtered.DiskDetective"
+            if coordinates or objectname:
+                service += ".Position"
+            mashupFilters = self._build_filter_set("Mast.Catalogs.Dd.Cone", service, **criteria)
+
+        else:
+            raise InvalidQueryError("Criteria query not availible for {}".format(catalog))
+
+        if not mashupFilters:
+            raise InvalidQueryError("At least one non-positional criterion must be supplied.")
+
+        if objectname and coordinates:
+            raise InvalidQueryError("Only one of objectname and coordinates may be specified.")
+
+        if objectname:
+            coordinates = self._resolve_object(objectname)
+
+        if coordinates:
+            # Put coordinates and radius into consitant format
+            coordinates = commons.parse_coordinates(coordinates)
+
+            # if radius is just a number we assume degrees
+            if isinstance(radius, (int, float)):
+                radius = radius * u.deg
+            radius = coord.Angle(radius)
+
+        # build query
+        if coordinates:
+            params = {"filters": mashupFilters,
+                      "ra": coordinates.ra.deg,
+                      "dec": coordinates.dec.deg,
+                      "radius": radius.deg}
+        else:
+            params = {"filters": mashupFilters}
+
+        # TIC needs columns specified
+        if catalog == "Tic":
+            params["columns"] = "c.*"
+
+        return self.service_request_async(service, params)
+
+    @class_or_instance
+    def query_hsc_matchid_async(self, match, pagesize=None, page=None):
+        """
+        Returns all the matches for a given Hubble Source Catalog MatchID.
+
+        Parameters
+        ----------
+        match : int or `~astropy.table.Row`
+            The matchID or HSC entry to return matches for.
+        pagesize : int, optional
+            Can be used to override the default pagesize.
+            E.g. when using a slow internet connection.
+        page : int, optional
+            Can be used to override the default behavior of all results being returned to obtain
+            one sepcific page of results.
+
+        Response
+        --------
+        response : list(`requests.Response`)
+        """
+
+        if isinstance(match, Row):
+            match = match["MatchID"]
+        match = str(match)  # np.int64 gives json serializer problems, so strigify right here
+
+        service = "Mast.HscMatches.Db"
+        params = {"input": match}
+
+        return self.service_request_async(service, params, pagesize, page)
+
+    @class_or_instance
+    def get_hsc_spectra_async(self, pagesize=None, page=None):
+        """
+        Returns all Hubble Source Catalog spectra.
+
+        Parameters
+        ----------
+        pagesize : int, optional
+            Can be used to override the default pagesize.
+            E.g. when using a slow internet connection.
+        page : int, optional
+            Can be used to override the default behavior of all results being returned to obtain
+            one sepcific page of results.
+
+        Response
+        --------
+        response : list(`requests.Response`)
+        """
+
+        service = "Mast.HscSpectra.Db.All"
+        params = {}
+
+        return self.service_request_async(service, params, pagesize, page)
+
+    def download_hsc_spectra(self, spectra, download_dir=None, cache=True, curl_flag=False):
+        """
+        Download one or more Hubble Source Catalog spectra.
+
+        Parameters
+        ----------
+        specrtra : `~astropy.table.Table` or `astropy.table.Row`
+            One or more HSC spectra to be downloaded.
+        download_dir : str, optional
+           Specify the base directory to download spectra into.
+           Spectra will be saved in the subdirectory download_dir/mastDownload/HSC.
+           If download_dir is not specified the base directory will be '.'.
+        cache : bool, optional
+            Default is True. If file is found on disc it will not be downloaded again.
+            Note: has no affect when downloading curl script.
+        curl_flag : bool, optional
+            Default is False.  If true instead of downloading files directly, a curl script
+            will be downloaded that can be used to download the data files at a later time.
+
+        Response
+        --------
+        response : list(`requests.Response`)
+        """
+
+        # if spectra is not a Table, put it in a list
+        if isinstance(spectra, Row):
+            spectra = [spectra]
+
+        # set up the download directory and paths
+        if not download_dir:
+            download_dir = '.'
+
+        if curl_flag:  # don't want to download the files now, just the curl script
+
+            downloadFile = "mastDownload_" + time.strftime("%Y%m%d%H%M%S")
+
+            urlList = []
+            pathList = []
+            for spec in spectra:
+                if spec['SpectrumType'] < 2:
+                    urlList.append('https://hla.stsci.edu/cgi-bin/getdata.cgi?config=ops&dataset=' +
+                                   spec['DatasetName'])
+
+                else:
+                    urlList.append('https://hla.stsci.edu/cgi-bin/ecfproxy?file_id=' +
+                                   spec['DatasetName'] + '.fits')
+
+                pathList.append(downloadFile + "/HSC/" + spec['DatasetName'] + '.fits')
+
+            descriptionList = [""]*len(spectra)
+            productTypeList = ['spectrum']*len(spectra)
+
+            service = "Mast.Bundle.Request"
+            params = {"urlList": ",".join(urlList),
+                      "filename": downloadFile,
+                      "pathList": ",".join(pathList),
+                      "descriptionList": list(descriptionList),
+                      "productTypeList": list(productTypeList),
+                      "extension": 'curl'}
+
+            response = self.service_request_async(service, params)
+            bundlerResponse = response[0].json()
+
+            localPath = download_dir.rstrip('/') + "/" + downloadFile + ".sh"
+            self._download_file(bundlerResponse['url'], localPath)
+
+            status = "COMPLETE"
+            msg = None
+            url = None
+
+            if not os.path.isfile(localPath):
+                status = "ERROR"
+                msg = "Curl could not be downloaded"
+                url = bundlerResponse['url']
+            else:
+                missingFiles = [x for x in bundlerResponse['statusList'].keys()
+                                if bundlerResponse['statusList'][x] != 'COMPLETE']
+                if len(missingFiles):
+                    msg = "{} files could not be added to the curl script".format(len(missingFiles))
+                    url = ",".join(missingFiles)
+
+            manifest = Table({'Local Path': [localPath],
+                              'Status': [status],
+                              'Message': [msg],
+                              "URL": [url]})
+
+        else:
+            base_dir = download_dir.rstrip('/') + "/mastDownload/HSC"
+
+            if not os.path.exists(base_dir):
+                os.makedirs(base_dir)
+
+            manifestArray = []
+            for spec in spectra:
+
+                # localPath = base_dir + "/HSC"# + spec['DatasetName'] + ".fits"
+
+                if spec['SpectrumType'] < 2:
+                    dataUrl = 'https://hla.stsci.edu/cgi-bin/getdata.cgi?config=ops&dataset=' \
+                              + spec['DatasetName']
+                else:
+                    dataUrl = 'https://hla.stsci.edu/cgi-bin/ecfproxy?file_id=' \
+                              + spec['DatasetName'] + '.fits'
+
+                localPath = base_dir + '/' + spec['DatasetName'] + ".fits"
+
+                status = "COMPLETE"
+                msg = None
+                url = None
+
+                try:
+                    self._download_file(dataUrl, localPath, cache=cache)
+
+                    # check file size also this is where would perform md5
+                    if not os.path.isfile(localPath):
+                        status = "ERROR"
+                        msg = "File was not downloaded"
+                        url = dataUrl
+
+                except HTTPError as err:
+                    status = "ERROR"
+                    msg = "HTTPError: {0}".format(err)
+                    url = dataUrl
+
+                manifestArray.append([localPath, status, msg, url])
+
+                manifest = Table(rows=manifestArray, names=('Local Path', 'Status', 'Message', "URL"))
+
+        return manifest
+
+
 Observations = ObservationsClass()
+Catalogs = CatalogsClass()
 Mast = MastClass()
